﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace SystemBankowy
{    
    class Bank
    {
        //struktura DDD
        public List<Konto> KontaROR = new List<Konto>(2);

        //funkcja DDD
        private static long WygenerujUnikalnyNumerKonta(List<Konto> konta)
        {
            if (konta.Count() == 0) return 0;
            return konta.Max(k => k.NumerKonta) + 1;
        }

        //funkcja DDD
        public static long TwórzKonto(List<Konto> konta, string właściciel)
        {
            long numerNowegoKonta = WygenerujUnikalnyNumerKonta(konta);
            Konto noweKonto = new Konto(numerNowegoKonta, 0, właściciel);
            konta.Add(noweKonto);
            return noweKonto.NumerKonta;
        }

        //procedura DDD
        public static bool WykonajPrzelew(Przelew poleceniePrzelewu)
        {
            try
            {
                poleceniePrzelewu.Nadawca.Wypłać(poleceniePrzelewu.Kwota);
                poleceniePrzelewu.Odbiorca.Wpłać(poleceniePrzelewu.Kwota);
                return true;
            }
            catch
            {                
                return false;
            }
        }
    }

    //obiekt DDD
    class Konto
    {        
        private long numerKonta;
        private decimal saldo;
        private string właściciel;

        public long NumerKonta
        {
            get
            {
                return numerKonta;
            }
        }

        public decimal Saldo
        {
            get
            {
                return saldo;
            }
        }

        public string Właściciel
        {
            get
            {
                return właściciel;
            }
        }

        public Konto(long numerKonta,decimal saldo,string właściciel)
        {
            this.numerKonta = numerKonta;
            this.saldo = saldo;
            this.właściciel = właściciel;
        }

        public void Wpłać(decimal kwota)
        {
            if(kwota<=0) throw new ArgumentException("Kwota nie może być mniejsza lub równa zeru");
            saldo+=kwota;
        }

        public void Wypłać(decimal kwota)
        {
            if(kwota<=0) throw new ArgumentException("Kwota nie może być mniejsza lub równa zeru");
            if(saldo-kwota<0) throw new ArgumentException("Nie ma możliwości zrobionia \"debetu\"");
            saldo-=kwota;
        }

        
    }

    //zadanie 2 (stworzyć funkcję DDD, która na podstawie FasadyPrzelewu stworzy przelew)
    struct FasadaPrzelewu
    {
        long numerKontaNadawcy;
        long numerKontaOdbiorcy;
        decimal kwota;
    }

    //struktura DDD
    struct Przelew
    {
        public Konto Nadawca;
        public Konto Odbiorca;
        public decimal Kwota;
    }


}
